var MdxDictController = JSB.defineClass('MdxDictController : UIViewController', {
  viewDidLoad: function() {
    self.navigationItem.title = 'MdxDict Addon Pannel';
    self.view.layer.shadowOffset = {width: 0, height: 0};
    self.view.layer.shadowRadius = 10;
    self.view.layer.shadowOpacity = 0.5;
    self.view.layer.shadowColor = UIColor.colorWithWhiteAlpha(0.5, 1);

    // >>> mdx dict view >>>
    self.mdxDictView = new UIWebView(self.view.bounds);
    self.mdxDictView.backgroundColor = UIColor.whiteColor();
    self.mdxDictView.scalesPageToFit = true;
    self.mdxDictView.autoresizingMask = (1 << 1 | 1 << 4);
    self.mdxDictView.delegate = self;
    self.mdxDictView.scrollView.delegate = self;
    self.mdxDictView.layer.cornerRadius = 15;
    self.mdxDictView.layer.masksToBounds = true;
    self.mdxDictView.loadFileURLAllowingReadAccessToURL(
      NSURL.fileURLWithPath(self.mainPath + '/mdict-js/mdict.html'),
      NSURL.fileURLWithPath(self.mainPath + '/mdict-js/')
    );
    self.view.addSubview(self.mdxDictView);
    // <<< mdx dict view <<<

    // >>> DeepL view >>>
    self.deeplView = new UIWebView(self.view.bounds);
    self.deeplView.url = (Application.sharedInstance().osType !== 1) ? 'https://www.deepl.com/translator' : 'https://www.deepl.com/translator-mobile'; 
    self.deeplView.backgroundColor = UIColor.whiteColor();
    self.deeplView.scalesPageToFit = true;
    self.deeplView.autoresizingMask = (1 << 1 | 1 << 4);
    self.deeplView.delegate = self;
    self.deeplView.scrollView.delegate = self;
    self.deeplView.layer.cornerRadius = 15;
    self.deeplView.layer.masksToBounds = true;
    self.deeplView.loadRequest(
      NSURLRequest.requestWithURL(NSURL.URLWithString(self.deeplView.url))
    );
    self.updateOffset();
    
    // >>> DeepL langauge button >>>
    self.deeplView.lanButton = UIButton.buttonWithType(0);
    self.deeplView.lanButton.autoresizingMask = (1 << 3);
    self.deeplView.lanButton.setTitleForState('Language', 0);
    self.deeplView.lanButton.setTitleColorForState(
      Application.sharedInstance().defaultTintColorForDarkBackground,
      0
    );
    var highlightColor = UIColor.blendedColor(
      Application.sharedInstance().defaultTintColorForDarkBackground,
      Application.sharedInstance().defaultTextColor,
      0.8
    );
    self.deeplView.lanButton.setTitleColorForState(highlightColor, 1);
    self.deeplView.lanButton.backgroundColor = 
      Application.sharedInstance().defaultTextColor.colorWithAlphaComponent(0.6);
    self.deeplView.lanButton.layer.cornerRadius = 10;
    self.deeplView.lanButton.layer.masksToBounds = true;
    self.deeplView.lanButton.titleLabel.font = UIFont.systemFontOfSize(14);
    self.deeplView.lanButton.addTargetActionForControlEvents(self, 'changeLanguage:', 1 << 6);
    self.deeplView.addSubview(self.deeplView.lanButton);
    // <<< DeepL langauge button <<<

    self.deeplView.hidden = true;
    self.deeplView.lanCode = NSUserDefaults.standardUserDefaults().objectForKey('mn_deepl_lan') || 'zh';
    self.deeplView.lastOffset = 0;
    self.updateButton();
    self.view.addSubview(self.deeplView);
    // <<< DeepL view <<<

    // >>> switch button >>>
    self.switchButton = UIButton.buttonWithType(0);
    self.switchButton.autoresizingMask = (1 << 0 | 1 << 3);
    self.switchButton.setTitleForState('🔄', 0);
    self.switchButton.setTitleColorForState(
      Application.sharedInstance().defaultTintColorForDarkBackground,
      0
    );
    self.switchButton.setTitleColorForState(highlightColor, 1);
    self.switchButton.backgroundColor =
      Application.sharedInstance().defaultTextColor.colorWithAlphaComponent(0.6);
    self.switchButton.layer.cornerRadius = 10;
    self.switchButton.layer.masksToBounds = true;
    self.switchButton.titleLabel.font = UIFont.systemFontOfSize(14);
    self.switchButton.addTargetActionForControlEvents(self, "switchButtonTapped:", 1 << 6);
    self.view.addSubview(self.switchButton);
    // <<< switch button <<<
  },
  viewWillAppear: function(animated) {
    self.mdxDictView.delegate = self;
    self.deeplView.delegate = self;
  },
  viewWillDisappear: function(animated) {
    self.mdxDictView.stopLoading();
    self.mdxDictView.delegate = null;
    self.deeplView.stopLoading();
    self.deeplView.delegate = null;
    UIApplication.sharedApplication().networkActivityIndicatorVisible = false;
  },
  viewWillLayoutSubviews: function() {
    var viewFrame = self.view.bounds;
    self.switchButton.frame = {
      x: viewFrame.x + viewFrame.width - 38,
      y: viewFrame.y + viewFrame.height - 38,
      width: 30,
      height: 30,
    };
    var deeplViewFrame = self.deeplView.bounds;
    self.deeplView.lanButton.frame = {
      x: viewFrame.x + 8,
      y: deeplViewFrame.y + deeplViewFrame.height - 38,
      width: 80,
      height: 30
    };
  },
  scrollViewDidScroll: function() {
  },
  webViewDidStartLoad: function(webView) {
    UIApplication.sharedApplication().networkActivityIndicatorVisible = true;
  },
  webViewDidFinishLoad: function(webView) {
    UIApplication.sharedApplication().networkActivityIndicatorVisible = false;
  },
  webViewDidFailLoadWithError: function(webView, error) {
    //UIApplication.sharedApplication().networkActivityIndicatorVisible = false;
    //var lan = NSLocale.preferredLanguages() ? NSLocale.preferredLanguages()[0].substring(0,2) : 'en';
    //if (lan == 'zh') {
    //  Application.sharedInstance().showHUD('DeepL加载失败，请检查网络后重试', self.view.window, 2);
    //} else {
    //  Application.sharedInstance().showHUD('Failed to load DeepL translator, please try again later', self.view.window, 2);
    //}
  },
  changeLanguage: function(sender) {
    if (self.deeplView.popoverController) {self.deeplView.popoverController.dismissPopoverAnimated(true);}
    var menuController = MenuController.new();
    menuController.commandTable = [
      {title:'English',object:self,selector:'changeLanguageTo:',param:'en',checked:(self.lanCode == 'en')},
      {title:'Chinese',object:self,selector:'changeLanguageTo:',param:'zh',checked:(self.lanCode == 'zh')},
      {title:'German',object:self,selector:'changeLanguageTo:',param:'de',checked:(self.lanCode == 'de')},
      {title:'French',object:self,selector:'changeLanguageTo:',param:'fr',checked:(self.lanCode == 'fr')},
      {title:'Spanish',object:self,selector:'changeLanguageTo:',param:'es',checked:(self.lanCode == 'es')},
      {title:'Russian',object:self,selector:'changeLanguageTo:',param:'ru',checked:(self.lanCode == 'ru')},
      {title:'Italian',object:self,selector:'changeLanguageTo:',param:'it',checked:(self.lanCode == 'it')},
      {title:'Japanese',object:self,selector:'changeLanguageTo:',param:'ja',checked:(self.lanCode == 'ja')}
    ];
    menuController.rowHeight = 35;
    menuController.preferredContentSize = {
      width: 100,
      height: menuController.rowHeight * menuController.commandTable.length
    };
    var studyController = Application.sharedInstance().studyController(self.view.window);
    self.deeplView.popoverController = new UIPopoverController(menuController);
    var r = sender.convertRectToView(sender.bounds,studyController.view);
    self.deeplView.popoverController.presentPopoverFromRect(r, studyController.view, 1 << 1, true);
  },
  changeLanguageTo: function(param) {
    self.deeplView.lanCode = param;
    self.updateButton();
    NSUserDefaults.standardUserDefaults().setObjectForKey(self.deeplView.lanCode,'mn_deepl_lan');
    self.deeplView.popoverController.dismissPopoverAnimated(true);
    if (self.text) {self.translate(self.text);}
  },
  switchButtonTapped: function() {
    self.switchView();
  }
});

MdxDictController.prototype.lookup = function(text) {
  this.text = text.replace(/^\s+/, '').replace(/\s+$/, '');
  this.mdxDictView.evaluateJavaScript(`$("#definition").empty();$("#word").val("${this.text}");$("#btn").click();`, res => {});
  this.translate(this.text);
  if (/^(\b\w+\b\s?){1,3}$/.test(this.text)) { // when text consists of up to 3 words, await lookup result
    NSTimer.scheduledTimerWithTimeInterval(0.1, false, () => {
      this.mdxDictView.evaluateJavaScript('document.lookupStatus;', res => {
        if (Number(res)&&this.mdxDictView.hidden || !Number(res)&&!this.mdxDictView.hidden) {this.switchView();} // switch to DeepL view when lookup fails
      });
    });
  } else { // switch to DeepL view when text consists of at least 4 words
    if (this.deeplView.hidden) {this.switchView();}
  }
};

MdxDictController.prototype.translate = function(text) {
  var url = `${this.deeplView.url}#en/${this.deeplView.lanCode}/${encodeURIComponent(text.replaceAll('/', '\\/'))}`;
  this.deeplView.loadRequest(NSURLRequest.requestWithURL(NSURL.URLWithString(url)));
  this.updateOffset();
};

MdxDictController.prototype.switchView = function() {
    this.mdxDictView.hidden = !this.mdxDictView.hidden;
    this.deeplView.hidden = !this.mdxDictView.hidden;
};

MdxDictController.prototype.updateButton = function() {
  if(this.deeplView.lanCode == 'en')
    this.deeplView.lanButton.setTitleForState('English',0);
  else if(this.deeplView.lanCode == 'zh')
    this.deeplView.lanButton.setTitleForState('Chinese',0);
  else if(this.deeplView.lanCode == 'ja')
    this.deeplView.lanButton.setTitleForState('Japanese',0);
  else if(this.deeplView.lanCode == 'de')
    this.deeplView.lanButton.setTitleForState('German',0);
  else if(this.deeplView.lanCode == 'fr')
    this.deeplView.lanButton.setTitleForState('French',0);
  else if(this.deeplView.lanCode == 'es')
    this.deeplView.lanButton.setTitleForState('Spanish',0);
  else if(this.deeplView.lanCode == 'ru')
    this.deeplView.lanButton.setTitleForState('Russian',0);
  else if(this.deeplView.lanCode == 'it')
    this.deeplView.lanButton.setTitleForState('Italian',0);
};

MdxDictController.prototype.updateOffset = function() {
  if(!this.deeplView || !this.deeplView.window)return;
  if (Application.sharedInstance().osType !== 1) {
    this.deeplView.evaluateJavaScript(
      `document.getElementsByClassName("dl_header")[0].style.display="none";
       document.getElementsByClassName("lmt__docTrans-tab-container")[0].style.display="none";
       document.getElementsByClassName("lmt__sides_container")[0].style.margin = 0;
       document.querySelector("#dl_translator").style.cssText = "padding-top: 20px";
       document.getElementsByClassName("lmt__language_container")[0].style.display = "none";
       document.getElementsByClassName("lmt__language_container")[1].style.display = "none";
       document.getElementsByClassName("lmt__target_toolbar")[0].style.display = "none";
       document.querySelector("#dl_cookieBanner").style.display="none";
       document.querySelector("#lmt_quotes_article").style.display="none";
       document.querySelector("#lmt__dict").style.margin = 0;
       document.querySelector("#lmt_pro_ad_container").style.display = "none";
       document.querySelector("body > div.dl_footerV2_container").style.display = "none";`,
      (ret) => {}
    );
  } else {
    this.deeplView.evaluateJavaScript(
      `document.querySelector("#gatsby-focus-wrapper > header").style.display = "none";
       document.querySelector("#gatsby-focus-wrapper > footer").style.display = "none";
       document.getElementsByClassName("TranslatorPage-module--otherSections--3cbQ7")[0].style.display = "none";`,
      (ret) => {}
    );
  }
  this.deeplView.evaluateJavaScript(
    'document.getElementsByClassName("lmt__side_container--target")[0].getBoundingClientRect().top + document.body.scrollTop+document.documentElement.scrollTop;',
    function (ret) {
      if (!this.deeplView) return;
      if (ret && !isNaN(parseFloat(ret))) {
        this.deeplView.scrollView.contentOffset = {x: 0, y: parseFloat(ret) + 0};          
      } else {
        this.deeplView.scrollView.contentOffset = {x: 0, y: 130};          
      }
    }
  );
};