JSB.newAddon = function(mainPath) {
  JSB.require('MdxDictController');
  var newAddonClass = JSB.defineClass(
    'MdxDictAddon : JSExtension',
    { /* Instance members */
      sceneWillConnect: function() { //Window initialize
          self.appInstance = Application.sharedInstance();
          self.addonController = MdxDictController.new();
          self.addonController.mainPath = mainPath;
          self.selectionText = '';
          self.rect = '{{0, 0}, {10, 10}}';
          self.arrow = 1;
      },

      sceneDidDisconnect: function() { // Window disconnect
      },

      sceneWillResignActive: function() { // Window resign active
      },

      sceneDidBecomeActive: function() { // Window become active
      },

      notebookWillOpen: function(notebookid) {
        if (self.appInstance.studyController(self.window).studyMode < 3 // Not support in card deck mode
            && NSUserDefaults.standardUserDefaults().objectForKey('MNAddon_MdxDict_On')) {
          self.appInstance.studyController(self.window).view.addSubview(self.addonController.view);
          self.addonController.view.hidden = true;
          NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onPopupMenuOnSelection:', 'PopupMenuOnSelection');
          NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onClosePopupMenuOnSelection:', 'ClosePopupMenuOnSelection');
          self.appInstance.studyController(self.window).refreshAddonCommands();
          NSTimer.scheduledTimerWithTimeInterval(0.2, false, function() { 
            self.appInstance.studyController(self.window).becomeFirstResponder(); //For dismiss keyboard on iOS
          });
        }
      },

      notebookWillClose: function(notebookid) {
        self.addonController.view.removeFromSuperview();
        NSNotificationCenter.defaultCenter().removeObserverName(self, 'PopupMenuOnSelection');
        NSNotificationCenter.defaultCenter().removeObserverName(self, 'ClosePopupMenuOnSelection');
      },

      documentDidOpen: function(docmd5) {
      },

      documentWillClose: function(docmd5) {
      },

      controllerWillLayoutSubviews: function(controller) {
        if (controller === self.appInstance.studyController(self.window)) {
            self.layoutAddonController();
        }
      },

      queryAddonCommandStatus: function() {
        if (self.appInstance.studyController(self.window).studyMode < 3) {
          return {
            image: 'MdxDict.png',
            object: self,
            selector: 'toggleAddon:',
            checked: self.addonController.view.window ? true : false
          };
        } else {
          return null;
        }
      },

      onPopupMenuOnSelection: function(sender) { // Selecting text on pdf or epub
        if (!self.appInstance.checkNotifySenderInWindow(sender,self.window)) return; // Don't process message from other window
        if (!self.addonController.view.window) return;
        if (self.viewTimer) self.viewTimer.invalidate();
        var text = sender.userInfo.documentController.selectionText;
        if (text && text.length) {
          self.addonController.lookup(text);
          self.layoutAddonController(sender.userInfo.winRect, sender.userInfo.arrow);
          self.addonController.view.hidden = false;
          self.selectionText = text;
        }
      },

      onPopupMenuOnNote: function(sender) { // Clicking note
      },

      onClosePopupMenuOnSelection: function(sender) {
        if (!sender.userInfo.documentController.selectionText) {
          self.addonController.view.hidden = true;
        } else {
          self.hideViewIfTextChanged();
        }
      },

      onClosePopupMenuOnNote: function(sender) {
      },

      toggleAddon: function(sender) {
        if (self.addonController.view.window) {
          NSUserDefaults.standardUserDefaults().setObjectForKey(false, 'MNAddon_MdxDict_On');
          NSNotificationCenter.defaultCenter().removeObserverName(self, 'PopupMenuOnSelection');
          NSNotificationCenter.defaultCenter().removeObserverName(self, 'ClosePopupMenuOnSelection');
          self.addonController.view.removeFromSuperview();
        } else {
          NSUserDefaults.standardUserDefaults().setObjectForKey(true, 'MNAddon_MdxDict_On');
          self.appInstance.studyController(self.window).view.addSubview(self.addonController.view);
          self.addonController.view.hidden = true;
          NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onPopupMenuOnSelection:', 'PopupMenuOnSelection');
          NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onClosePopupMenuOnSelection:', 'ClosePopupMenuOnSelection');
          NSTimer.scheduledTimerWithTimeInterval(0.2, false, function() { 
            self.appInstance.studyController(self.window).becomeFirstResponder(); // For dismiss keyboard on iOS
          });
        }
        self.appInstance.studyController(self.window).refreshAddonCommands();
      },
    },
    { /* Class members */
      addonDidConnect: function() {
      },

      addonWillDisconnect: function() {
      },

      applicationWillEnterForeground: function() {
      },

      applicationDidEnterBackground: function() {
      },

      applicationDidReceiveLocalNotification: function(notify) {
      }
    }
  );
  
  newAddonClass.prototype.hideViewIfTextChanged = function() {
    this.viewTimer = NSTimer.scheduledTimerWithTimeInterval(0.1, true, () => {
      var currText = this.appInstance.studyController(this.window).readerController.currentDocumentController.selectionText;
      if (!currText || currText !== this.selectionText) {
        this.addonController.view.hidden = true;
        this.viewTimer.invalidate();
      }
    });
  };

  newAddonClass.prototype.layoutAddonController = function(rectStr, arrowNum) {
    this.rect = rectStr || this.rect;
    this.arrow = arrowNum || this.arrow;
    var x, y
        w = (this.appInstance.osType !== 1) ? 620 : 365, // this.addonController.view.frame.width
        h = 650, // this.addonController.view.frame.height
        fontSize = 15,
        margin = 10,
        padding = 20,
        frame = this.appInstance.studyController(this.window).view.bounds,
        W = frame.width,
        H = frame.height,
        rectArr = this.rect.replace(/{/g, '').replace(/}/g, '').replace(/\s/g, '').split(','),
        X = Number(rectArr[0]),
        Y = Number(rectArr[1]),
        studyMode = this.appInstance.studyController(this.window).studyMode,
        contextMenuWidth = studyMode === 0 ? 225 : 435,
        contextMenuHeight = 35,
        textMenuPadding = 40;

    // this.addonController.view.frame.x
    if (w >= contextMenuWidth) {
      if (X - w/2 - margin <= 0) {
        x = margin;
      } else if (X + w/2 + margin >= W) {
        x = W - margin - w;
      } else {
        x = X - w/2;
      }
    } else {
      if (X - contextMenuWidth/2 - margin <= 0) {
        x = margin + contextMenuWidth/2 - w/2;
      } else if (X + contextMenuWidth/2 + margin >= W) {
        x = W - margin - contextMenuWidth/2 - w/2;
      } else {
        x = X - w/2;
      }
    }
    
    // this.addonController.view.frame.[y, height]
    if (this.arrow === 1) {
      let upperBlankHeight = Y - textMenuPadding - fontSize - padding,
          lowerBlankHeight = H - Y - contextMenuHeight - padding;
      if (upperBlankHeight >= lowerBlankHeight) {
        h = (upperBlankHeight >= h) ? h : upperBlankHeight;
        y = upperBlankHeight - h;
      } else {
        y = H - lowerBlankHeight;
        h = (H - y >= h) ? h : H - y;
      }
    } else {
      let upperBlankHeight = Y - textMenuPadding - contextMenuHeight - padding,
          lowerBlankHeight = H - Y - fontSize - padding;
      if (upperBlankHeight >= lowerBlankHeight) {
        h = (upperBlankHeight >= h) ? h : upperBlankHeight;
        y = upperBlankHeight - h; 
      } else {
        y = H - lowerBlankHeight;
        h = (H - y >= h) ? h : H - y;
      }
    }

    this.addonController.view.frame = {x: x, y: y, width: w, height: h};
  };

  return newAddonClass;
};